import requests
import json
from datetime import datetime

import pysnowball.token as token


def fetch(url, host="stock.xueqiu.com"):
    HEADERS = {'Host': host,
               'Accept': 'application/json',
               'Cookie': token.get_token(),
               'User-Agent': 'Xueqiu iPhone 11.8',
               'Accept-Language': 'zh-Hans-CN;q=1, ja-JP;q=0.9',
               'Accept-Encoding': 'br, gzip, deflate',
               'Connection': 'keep-alive'}

    response = requests.get(url, headers=HEADERS)

    # print(url)
    # print(HEADERS)
    # print(response)
    # print(response.content)

    if response.status_code != 200:
        raise Exception(response.content)

    return json.loads(response.content)


def fetch_without_token(url, host="stock.xueqiu.com"):
    HEADERS = {'Host': host,
               'Accept': 'application/json',
               'User-Agent': 'Xueqiu iPhone 11.8',
               'Accept-Language': 'zh-Hans-CN;q=1, ja-JP;q=0.9',
               'Accept-Encoding': 'br, gzip, deflate',
               'Connection': 'keep-alive'}

    response = requests.get(url, headers=HEADERS)

    # print(url)
    # print(HEADERS)
    # print(response)
    # print(response.content)

    if response.status_code != 200:
        raise Exception(response.content)

    return json.loads(response.content)


def fetch_eastmoney(url):
    HEADERS = {"Host": "datacenter-web.eastmoney.com",
               "User-Agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/96.0.4664.110 Safari/537.36",
               "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9",
               "Accept-Encoding": "gzip, deflate, br",
               "Accept-Language": "en-US,en;q=0.9,zh-CN;q=0.8,zh;q=0.7,cy;q=0.6"}
    response = requests.get(url, headers=HEADERS)

    if response.status_code != 200:
        raise Exception(response.content)

    return json.loads(response.content)


def fetch_csindex(url):
    HEADERS = {"Host": "www.csindex.com.cn",
               "User-Agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/96.0.4664.110 Safari/537.36",
               "Accept": "application/json, text/plain, */*",
               "Accept-Encoding": "gzip, deflate, br",
               "Accept-Language": "en-US,en;q=0.9,zh-CN;q=0.8,zh;q=0.7,cy;q=0.6"}
    response = requests.get(url, headers=HEADERS)
    
    # print(url)
    # print(HEADERS)
    # print(response)
    # print(response.content)

    if response.status_code != 200:
        raise Exception(response.content)

    return json.loads(response.content)


def fetch_hkc(url, txt_date=None):
    today = datetime.today()
    if txt_date is None:
        txt_date = today.strftime('%Y/%m/%d')
    today_str = today.strftime('%Y%m%d')
    payload = {
        '__VIEWSTATE': '/wEPDwUJNjIxMTYzMDAwZGSFj8kdzCLeVLiJkFRvN5rjsPotqw==',
        '__VIEWSTATEGENERATOR': '3C67932C',
        '__EVENTVALIDATION': '/wEdAAdbi0fj+ZSDYaSP61MAVoEdVobCVrNyCM2j+bEk3ygqmn1KZjrCXCJtWs9HrcHg6Q64ro36uTSn/Z2SUlkm9HsG7WOv0RDD9teZWjlyl84iRMtpPncyBi1FXkZsaSW6dwqO1N1XNFmfsMXJasjxX85ju3P1WAPUeweM/r0/uwwyYLgN1B8=',
        'today': today_str,
        'sortBy': 'stockcode',
        'sortDirection': 'asc',
        'alertMsg': '',
        'txtShareholdingDate': txt_date,
        'btnSearch': 'Search'
    }
    # payload = parse.urlencode(payload)
    headers = {
        'Content-Type': 'application/x-www-form-urlencoded'
    }
    
    requests.packages.urllib3.disable_warnings()
    response = requests.post(url, headers=headers, data=payload, verify=False)
    
    # print(url)
    # print(response)
    # print(response.content)

    if response.status_code != 200:
        raise Exception(response.content)
    return response.content


def fetch_danjuan_fund(url):
    fund_header = {
        'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/92.0.4515.107 Safari/537.36',
    }

    response = requests.request(method="GET", url=url, headers=fund_header)

    if response.status_code != 200:
        raise Exception(response.content)

    return response.json()



import json
import os
from pysnowball import cons
from pysnowball import api_ref
from pysnowball import utls


def margin(symbol, page=1, size=180):
    url = api_ref.capital_margin_url+symbol
    url = url + '&page='+str(page)
    url = url + '&size='+str(size)
    return utls.fetch(url)


def blocktrans(symbol, page=1, size=30):
    url = api_ref.capital_blocktrans_url+symbol
    url = url + '&page='+str(page)
    url = url + '&size='+str(size)
    return utls.fetch(url)


def capital_assort(symbol):
    url = api_ref.capital_assort_url+symbol
    return utls.fetch(url)


def capital_flow(symbol):
    url = api_ref.capital_flow_url+symbol
    return utls.fetch(url)


def capital_history(symbol, count=20):
    url = api_ref.capital_history_url+symbol
    url = url + '&count='+str(count)
    return utls.fetch(url)



import json
import os
from pysnowball import cons
from pysnowball import api_ref
from pysnowball import utls


def report(symbol):
    url = api_ref.report_latest_url+symbol
    return utls.fetch(url)


def earningforecast(symbol):
    url = api_ref.report_earningforecast_url+symbol
    return utls.fetch(url)



import json
import os
from pysnowball import cons
from pysnowball import api_ref
from pysnowball import utls


def skholderchg(symbol):
    url = api_ref.f10_skholderchg+symbol
    return utls.fetch(url)


def skholder(symbol):
    url = api_ref.f10_skholder+symbol
    return utls.fetch(url)


def industry(symbol):
    url = api_ref.f10_industry+symbol
    return utls.fetch(url)


def holders(symbol):
    url = api_ref.f10_holders+symbol
    return utls.fetch(url)


def bonus(symbol,page=1,size=10):
    url = api_ref.f10_bonus+symbol
    url = url + '&page='+str(page)
    url = url + '&size='+str(size)
    return utls.fetch(url)


def org_holding_change(symbol):
    url = api_ref.f10_org_holding_change+symbol
    return utls.fetch(url)


def industry_compare(symbol):
    url = api_ref.f10_industry_compare+symbol
    return utls.fetch(url)


def business_analysis(symbol):
    url = api_ref.f10_business_analysis+symbol
    return utls.fetch(url)


def shareschg(symbol, count=5):
    url = api_ref.f10_shareschg+symbol
    url = url + '&count='+str(count)
    return utls.fetch(url)


def top_holders(symbol, circula=1):
    url = api_ref.f10_top_holders+symbol
    url = url + '&circula='+str(circula)
    return utls.fetch(url)


def main_indicator(symbol):
    url = api_ref.f10_indicator+symbol
    return utls.fetch(url)



from unittest import TestCase

from pysnowball import fund

test_fund_code = "008975"


class Test(TestCase):
    def test_fund_detail(self):
        fund_detail = fund.fund_detail(test_fund_code)
        print(fund_detail)

    def test_fund_info(self):
        fund_info = fund.fund_info(test_fund_code)
        print(fund_info)

    def test_fund_growth(self):
        fund_growth = fund.fund_growth(test_fund_code)
        print(fund_growth)

    def test_fund_nav_history(self):
        history = fund.fund_nav_history(test_fund_code)
        print(history)

    def test_fund_achievement(self):
        fund_achievement = fund.fund_achievement(test_fund_code)
        print(fund_achievement)

    def test_fund_asset(self):
        fund_asset = fund.fund_asset(test_fund_code)
        print(fund_asset)

    def test_fund_manager(self):
        fund_manager = fund.fund_manager(test_fund_code)
        print(fund_manager)

    def test_fund_trade_date(self):
        fund_trade_date = fund.fund_trade_date(test_fund_code)
        print(fund_trade_date)

    def test_fund_derived(self):
        fund_derived = fund.fund_derived(test_fund_code)
        print(fund_derived)



from datetime import datetime
from datetime import timedelta
from pysnowball import api_ref
from pysnowball import utls


def index_basic_info(symbols):
    url = api_ref.index_basic_info.format(symbols)
    return utls.fetch_csindex(url)


def index_details_data(symbols):
    url = api_ref.index_details_data.format(symbols)
    return utls.fetch_csindex(url)


def index_weight_top10(symbols):
    url = api_ref.index_weight_top10.format(symbols)
    return utls.fetch_csindex(url)


def index_perf_7(symbols):
    today = datetime.now()
    firstday = today - timedelta(days=7)
    url = api_ref.index_perf.format(symbols, datetime.strftime(
        firstday, "%Y%m%d"), datetime.strftime(today, "%Y%m%d"))
    return utls.fetch_csindex(url)


def index_perf_30(symbols):
    today = datetime.now()
    firstday = today - timedelta(days=30)
    url = api_ref.index_perf.format(symbols, datetime.strftime(
        firstday, "%Y%m%d"), datetime.strftime(today, "%Y%m%d"))
    return utls.fetch_csindex(url)


def index_perf_90(symbols):
    today = datetime.now()
    firstday = today - timedelta(days=90)
    url = api_ref.index_perf.format(symbols, datetime.strftime(
        firstday, "%Y%m%d"), datetime.strftime(today, "%Y%m%d"))
    return utls.fetch_csindex(url)



from pysnowball import api_ref
from pysnowball import utls

host = "xueqiu.com"

def nav_daily(symbol):
    url = api_ref.nav_daily + symbol
    return utls.fetch(url, host)

def rebalancing_history(symbol, count=20, page=1):
    url = api_ref.rebalancing_history + symbol
    url = url + "&count=" + str(count)
    url = url + "&page=" + str(page)
    return utls.fetch(url, host)


import json
import os
from pysnowball import cons
from pysnowball import api_ref
from pysnowball import utls



def cash_flow(symbol, is_annals=0, count=10):

    url = api_ref.finance_cash_flow_url+symbol
    
    if is_annals == 1:
        url = url + '&type=Q4'
    
    url = url + '&count='+str(count)

    return utls.fetch(url)


def indicator(symbol, is_annals=0, count=10):
    
    url = api_ref.finance_indicator_url+symbol
    
    if is_annals == 1:
        url = url + '&type=Q4'
    
    url = url + '&count='+str(count)

    return utls.fetch(url)


def balance(symbol, is_annals=0, count=10):

    url = api_ref.finance_balance_url+symbol

    if is_annals == 1:
        url = url + '&type=Q4'

    url = url + '&count='+str(count)

    return utls.fetch(url)


def income(symbol, is_annals=0, count=10):
    
    url = api_ref.finance_income_url+symbol

    if is_annals == 1:
        url = url + '&type=Q4'

    url = url + '&count='+str(count)

    return utls.fetch(url)


def business(symbol, is_annals=0, count=10):

    url = api_ref.finance_business_url+symbol

    if is_annals == 1:
        url = url + '&type=Q4'

    url = url + '&count='+str(count)

    return utls.fetch(url)
 



import os

name = "pysnowball"

__author__ = 'Yang Yu'


from pysnowball.finance import (cash_flow, indicator, balance, income, business)

from pysnowball.report import (report, earningforecast)

from pysnowball.capital import(
    margin, blocktrans, capital_assort, capital_flow, capital_history)

from pysnowball.realtime import(quotec, pankou, quote_detail, kline)

from pysnowball.f10 import(skholderchg, skholder, main_indicator,
                           industry, holders, bonus, org_holding_change,
                           industry_compare, business_analysis, shareschg, top_holders)

from pysnowball.token import (get_token,set_token)

from pysnowball.user import(watch_list, watch_stock)

from pysnowball.cube import(nav_daily, rebalancing_history)

from pysnowball.bond import(convertible_bond)

from pysnowball.index import(index_basic_info, index_details_data, index_weight_top10,
                             index_perf_7, index_perf_30, index_perf_90)

from pysnowball.hkex import(
    northbound_shareholding_sh, northbound_shareholding_sz)

from pysnowball.fund import (fund_detail, fund_info, fund_growth,
                             fund_nav_history, fund_derived, fund_asset,
                             fund_manager, fund_achievement, fund_trade_date)




from pysnowball import api_ref
from pysnowball import utls


def fund_detail(fund_code):
    return utls.fetch_danjuan_fund(api_ref.fund_detail % fund_code)


def fund_info(fund_code):
    return utls.fetch_danjuan_fund(api_ref.fund_info % fund_code)


def fund_growth(fund_code, day='ty'):
    return utls.fetch_danjuan_fund(api_ref.fund_growth % (fund_code, day))


def fund_nav_history(fund_code, page=1, size=10):
    return utls.fetch_danjuan_fund(api_ref.fund_nav_history % (fund_code, page, size))


def fund_achievement(fund_code):
    return utls.fetch_danjuan_fund(api_ref.fund_achievement % fund_code)


def fund_asset(fund_code):
    return utls.fetch_danjuan_fund(api_ref.fund_asset % fund_code)


def fund_manager(fund_code, post_status=1):
    return utls.fetch_danjuan_fund(api_ref.fund_manager % (fund_code, post_status))


def fund_trade_date(fund_code):
    return utls.fetch_danjuan_fund(api_ref.fund_trade_date % fund_code)


def fund_derived(fund_code):
    return utls.fetch_danjuan_fund(api_ref.fund_derived % fund_code)



# -*- coding:utf-8 -*-
from __future__ import print_function



import json
import os
import logging
from bs4 import BeautifulSoup

from pysnowball import utls
from pysnowball import api_ref

logger = logging.getLogger(__name__)
logger.setLevel(logging.ERROR)


def northbound_shareholding_sh(txt_date=None):
    """
    date: '%Y/%m/%d'
    """
    return _get_shareholding('sh', txt_date)


def northbound_shareholding_sz(txt_date=None):
    """
    date: '%Y/%m/%d'
    """
    return _get_shareholding('sz', txt_date)


def _get_shareholding(exchange, txt_date):
    try:
        html = utls.fetch_hkc(api_ref.hkex_connect + exchange.lower(), txt_date)
        bsObj = BeautifulSoup(html, "html.parser")
        data = []
        for tr in bsObj.find("table", {"id": "mutualmarket-result"}).find("tbody").findAll("tr"):
            code = tr.find("td", {"class": "col-stock-code"}
                           ).find("div", {"class": "mobile-list-body"}).get_text()
            name = tr.find("td", {"class": "col-stock-name"}
                           ).find("div", {"class": "mobile-list-body"}).get_text()
            shareholding = tr.find("td", {"class": "col-shareholding"}
                                   ).find("div", {"class": "mobile-list-body"}).get_text()
            shareholding_percent = tr.find("td", {"class": "col-shareholding-percent"}
                                           ).find("div", {"class": "mobile-list-body"}).get_text()
            shareholding = int(shareholding.replace(',',''))
            data.append({
                'code':code,
                'name':name,
                'shareholding':shareholding,
                'shareholding_percent':shareholding_percent
            })
        return data
    except Exception as e:
        logger.error(e)


# | from | 70 | 71 | 72 | 73 | 77 | 78 | 90 | 91 | 93 | 95 | 30 |
# | to   |000 |001 |002 |003 |300 |301 |600 |601 |603 |605 |688 |

def tran_code(code):
    code_str = str(code)
    d2 = code_str[0:2]
    d3 = code_str[2:5]

    if d2 == "70":
        return 'SZ000'+d3
    elif d2 == "71":
        return 'SZ001'+d3
    elif d2 == "72":
        return 'SZ002'+d3
    elif d2 == "73":
        return 'SZ003'+d3
    elif d2 == "77":
        return 'SZ300'+d3
    elif d2 == "78":
        return 'SZ301'+d3
    elif d2 == "90":
        return 'SH600'+d3
    elif d2 == "91":
        return 'SH601'+d3
    elif d2 == "92":
        return 'SH602'+d3
    elif d2 == "93":
        return 'SH603'+d3
    elif d2 == "94":
        return 'SH604'+d3
    elif d2 == "95":
        return 'SH605'+d3
    elif d2 == "30":
        return 'SH688'+d3
    else:
        logger.error('tran_code error:'+code_str)
        raise('tran_code error:'+code_str)




import time
from pysnowball import cons
from pysnowball import api_ref
from pysnowball import utls


def quotec(symbols):
    url = api_ref.realtime_quote+symbols
    return utls.fetch_without_token(url)


def quote_detail(symbol):
    return utls.fetch(api_ref.realtime_quote_detail+symbol)


def pankou(symbol):
    url = api_ref.realtime_pankou+symbol
    return utls.fetch(url)


def kline(symbol, days=100):
    return utls.fetch(api_ref.kline.format(symbol, int(time.time()*1000), days))



from pysnowball import api_ref
from pysnowball import utls

def watch_list():
    url = api_ref.watch_list
    return utls.fetch(url)

def watch_stock(id):
    url = api_ref.watch_stock + str(id)
    return utls.fetch(url)


import os
import pysnowball.cons as cons

def get_token():
    if os.environ.get('XUEQIUTOKEN') is None:
        raise Exception(cons.NOTOKEN_ERROR_MSG)
    else:
        return os.environ['XUEQIUTOKEN']

def set_token(token):
    os.environ['XUEQIUTOKEN'] = token
    return os.environ['XUEQIUTOKEN']



import json
import os
from pysnowball import api_ref
from pysnowball import utls

# EastMoney RPT_BOND_CB_LIST


def convertible_bond(page_size, page_count):
    url = api_ref.convertible_bond.format(page_size, page_count)
    return utls.fetch_eastmoney(url)




# finance
finance_cash_flow_url = "https://stock.xueqiu.com/v5/stock/finance/cn/cash_flow.json?symbol="
finance_indicator_url = "https://stock.xueqiu.com/v5/stock/finance/cn/indicator.json?symbol="
finance_balance_url = "https://stock.xueqiu.com/v5/stock/finance/cn/balance.json?symbol="
finance_income_url = "https://stock.xueqiu.com/v5/stock/finance/cn/income.json?symbol="
finance_business_url = "https://stock.xueqiu.com/v5/stock/finance/cn/business.json?symbol="

# report
report_latest_url = "https://stock.xueqiu.com/stock/report/latest.json?symbol="
report_earningforecast_url = "https://stock.xueqiu.com/stock/report/earningforecast.json?symbol="

# capital
capital_margin_url = "https://stock.xueqiu.com/v5/stock/capital/margin.json?symbol="
capital_blocktrans_url = "https://stock.xueqiu.com/v5/stock/capital/blocktrans.json?symbol="
capital_assort_url = "https://stock.xueqiu.com/v5/stock/capital/assort.json?symbol="
capital_history_url = "https://stock.xueqiu.com/v5/stock/capital/history.json?symbol="
capital_flow_url = "https://stock.xueqiu.com/v5/stock/capital/flow.json?symbol="

# f10
f10_skholderchg = "https://stock.xueqiu.com/v5/stock/f10/cn/skholderchg.json?symbol="
f10_skholder = "https://stock.xueqiu.com/v5/stock/f10/cn/skholder.json?symbol="
f10_industry = "https://stock.xueqiu.com/v5/stock/f10/cn/industry.json?symbol="
f10_holders = "https://stock.xueqiu.com/v5/stock/f10/cn/holders.json?&symbol="
f10_bonus = "https://stock.xueqiu.com/v5/stock/f10/cn/bonus.json?&symbol="
f10_org_holding_change = "https://stock.xueqiu.com/v5/stock/f10/cn/org_holding/change.json?symbol="
f10_industry_compare = "https://stock.xueqiu.com/v5/stock/f10/cn/industry/compare.json?type=single&symbol="
f10_business_analysis = "https://stock.xueqiu.com/v5/stock/f10/cn/business_analysis.json?symbol="
f10_shareschg = "https://stock.xueqiu.com/v5/stock/f10/cn/business_analysis.json?symbol="
f10_top_holders = "https://stock.xueqiu.com/v5/stock/f10/cn/top_holders.json?&symbol="
f10_indicator = "https://stock.xueqiu.com/v5/stock/f10/cn/indicator.json?symbol="

# real time
realtime_quote = "https://stock.xueqiu.com/v5/stock/realtime/quotec.json?symbol="
realtime_pankou = "https://stock.xueqiu.com/v5/stock/realtime/pankou.json?symbol="
realtime_quote_detail = "https://stock.xueqiu.com/v5/stock/quote.json?extend=detail&symbol="

# kline

kline = "https://stock.xueqiu.com/v5/stock/chart/kline.json?symbol={}&begin={}&period=day&type=before&count=-{}&indicator=kline,pe,pb,ps,pcf,market_capital,agt,ggt,balance"

# user
watch_list = "https://stock.xueqiu.com/v5/stock/portfolio/list.json?system=true"
watch_stock = "https://stock.xueqiu.com/v5/stock/portfolio/stock/list.json?size=1000&category=1&pid="

# cube
nav_daily = "https://xueqiu.com/cubes/nav_daily/all.json?cube_symbol="
rebalancing_history = "https://xueqiu.com/cubes/rebalancing/history.json?cube_symbol="

# eastmoney
convertible_bond = "https://datacenter-web.eastmoney.com/api/data/v1/get?pageSize={}&pageNumber={}&sortColumns=PUBLIC_START_DATE&sortTypes=-1&reportName=RPT_BOND_CB_LIST&columns=ALL&quoteColumns=f2~01~CONVERT_STOCK_CODE~CONVERT_STOCK_PRICE%2Cf235~10~SECURITY_CODE~TRANSFER_PRICE%2Cf236~10~SECURITY_CODE~TRANSFER_VALUE%2Cf2~10~SECURITY_CODE~CURRENT_BOND_PRICE%2Cf237~10~SECURITY_CODE~TRANSFER_PREMIUM_RATIO%2Cf239~10~SECURITY_CODE~RESALE_TRIG_PRICE%2Cf240~10~SECURITY_CODE~REDEEM_TRIG_PRICE%2Cf23~01~CONVERT_STOCK_CODE~PBV_RATIO&source=WEB&client=WEB"

# csindex
index_basic_info = "https://www.csindex.com.cn/csindex-home/indexInfo/index-basic-info/{}"
index_details_data = "https://www.csindex.com.cn/csindex-home/indexInfo/index-details-data?fileLang=1&indexCode={}"
index_weight_top10 = "https://www.csindex.com.cn/csindex-home/index/weight/top10/{}"
index_perf = "https://www.csindex.com.cn/csindex-home/perf/index-perf?indexCode={}&startDate={}&endDate={}"

# hkex
hkex_connect = "http://www.hkexnews.hk/sdw/search/mutualmarket.aspx?t="

# fund
# param is fund code
fund_detail = "https://danjuanapp.com/djapi/fund/detail/%s"
# param is fund code
fund_info = "https://danjuanapp.com/djapi/fund/%s"
# first param is fund code, second is 'ty'
fund_growth = "https://danjuanapp.com/djapi/fund/growth/%s?day=%s"
# first param is fund code
fund_nav_history = "https://danjuanapp.com/djapi/fund/nav/history/%s?page=%s&size=%s"
# param is fund code
fund_achievement = "https://danjuanapp.com/djapi/fundx/base/fund/achievement/%s"
# 基金持仓：param is fund code
fund_asset = "https://danjuanapp.com/djapi/fundx/base/fund/record/asset/percent?fund_code=%s"
# 基金管理人: param is fund code
fund_manager = "https://danjuanapp.com/djapi/fundx/base/fund/record/manager/list?fund_code=%s&post_status=%s"
# https://danjuanapp.com/djapi/fund/base/quote/data/index/analysis/008975
# param is fund code
fund_trade_date = "https://danjuanapp.com/djapi/fund/order/v2/trade_date?fd_code=%s"
# param is fund code
fund_derived = "https://danjuanapp.com/djapi/fund/derived/%s"

